﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using System;

public class PathRenderer : MonoBehaviour
{
    [SerializeField]
    Square square;
    [SerializeField]
    GameObject PathCollider;

    LineRenderer _pathRenderer;
    float _zPosition => square.transform.position.z;
    float _originalPathColliderY;
    // Start is called before the first frame update
    void Start()
    {
        _originalPathColliderY = PathCollider.transform.localScale.y;

        _pathRenderer = GetComponent<LineRenderer>();
        _ResetPaths();
    }

    // Update is called once per frame
    void Update()
    {
        if (Input.GetMouseButtonDown(1))
        {
            RaycastHit hit;
            var ray = Camera.main.ScreenPointToRay(Input.mousePosition);

            if (Physics.Raycast(ray, out hit))
            {
                if (Input.GetKey(KeyCode.LeftShift))
                {
                    Debug.Log("asdf");
                    _DrawMorePath(hit.point);
                }
                else
                {
                    _DrawOnePath(hit.point);
                }
            }
        }

        if (Input.GetMouseButtonDown(0))
        {
            RaycastHit hit;
            var ray = Camera.main.ScreenPointToRay(Input.mousePosition);

            if (Physics.Raycast(ray, out hit))
            {
                Debug.Log(hit.collider.gameObject.name);
            }
        }
    }

    private void _DrawOnePath(Vector3 point)
    {
        _ResetPaths();
        point.z = _zPosition;
        square.pathList[0] = transform.position;
        square.pathList[1] = point;
        _pathRenderer.SetPositions(square.pathList.ToArray());
        _modifyCapsule();
    }

    private void _ResetPaths()
    {
        _pathRenderer.positionCount = 2;
        square.pathList.Clear();
        square.pathList.Add(transform.position);
        square.pathList.Add(new Vector3(0, 0, _zPosition));
        _pathRenderer.SetPositions(square.pathList.ToArray());
    }

    private void _DrawMorePath(Vector3 point)
    {
        point.z = _zPosition;
        square.pathList.Add(point);
        _pathRenderer.positionCount = square.pathList.Count();
        _pathRenderer.SetPositions(square.pathList.ToArray());
    }

    private void _modifyCapsule()
    {
        PathCollider.transform.localScale = new Vector3(2, _originalPathColliderY, 0);
        PathCollider.transform.eulerAngles = new Vector3(0, 0,
            (float)Constants.RadianToDegree(Mathf.PI / 2 +
                Mathf.Atan
                (square.GetTangent(
                    square.GetNthPath(0)))));

        float _newY = square.GetNthPath(0).magnitude;
        PathCollider.transform.localScale = new Vector3(2, _newY * _originalPathColliderY, 0);

        PathCollider.transform.position = square.GetNthPath(0) / 2;
    }
}
